;AssembleIt by Andy @ www.autoit.de
;
;see examples how to call _AssembleIt()

;Debugger included



#include-once
#include <FASM.au3>
#include <ButtonConstants.au3>
#include <EditConstants.au3>
#include <GUIConstantsEx.au3>
;#include <GUIListBox.au3>
#include <GuiStatusBar.au3>
#include <StaticConstants.au3>
#include <WindowsConstants.au3>
#include <Constants.au3>
#include <array.au3>
#include <GuiListView.au3>
#include <winapi.au3>




If @AutoItX64 Then
    MsgBox(0, "_AssembleIt Error", "Sorry, 64Bit is not supported. Program will be terminated!")
    Exit
EndIf

Global $_ASSEMBLEIT_FLAG = 1
Global $Fasm = FasmInit()
If @error Then
    MsgBox(0, "_AssembleIt Error", "Not able to FasmInit! Program will be terminated!")
    Exit
EndIf

; #FUNCTION# ======================================================================================
; Name ..........: _AssembleIt()
; Description ...: "Wrapper" for the FASM.au3 by Ward
; Syntax ........: _AssembleIt($Returntype, $sFunc, $Type1 = "type", $Param1 = 0, $Type2 = "type", $Param2 = 0....
; Parameters ....: $Returntype          - Data type returned by the assembled program
;                  $sFunc               - Name of the function, in which the Assemblercode is contained
;                  $sType1              - DataType of Parameter1
;                  $sParam1             - Parameter1
;                  $sType2              - DataType of Parameter2
;                  $sParam2             - Parameter2.....and so on, you can pass up to 20 parameters
;
; Return values .: Success    depends on $Returntype    @error=0
;                  Failure    @error = -2   FasmReset has failed
;                  Failure    @error = -3   Error in Assemblercode detected by Fasm
;                  Failure    @error = -4   Function with Assemblercode doesnt exist (i.e. wrong functionname)
;                  Failure    @error = -5   Error while executing MemoryFuncCall
;
; Author ........: Andy @ www.autoit.de
; Modified ......:
; Remarks .......: _AssembleIt() instructs MemoryFuncCall with cdecl-convention, so only a RET is necessary at the end of the ASM-code
;                  If $_ASSEMBLEIT_FLAG = 0 is set before calling AssembleIt(), an AutoIt-code to call the opcodes without the need of FASM.au3 is created
; Related .......: Fasm.au3 by Ward  http://www.autoitscript.com/forum/index.php?showtopic=111613&view=findpost&p=782727
; Link ..........:
; Example .......:
; =================================================================================================

Func _AssembleIt($Returntype, $sFunc, $Type1 = "int", $Param1 = 0, $Type2 = "int", $Param2 = 0, $Type3 = "int", $Param3 = 0, $Type4 = "int", $Param4 = 0, $Type5 = "", $Param5 = 0, $Type6 = "", $Param6 = 0, $Type7 = "", $Param7 = 0, $Type8 = "", $Param8 = 0, $Type9 = "", $Param9 = 0, $Type10 = "", $Param10 = 0, $Type11 = "", $Param11 = 0, $Type12 = "", $Param12 = 0, $Type13 = "", $Param13 = 0, $Type14 = "", $Param14 = 0, $Type15 = "", $Param15 = 0, $Type16 = "", $Param16 = 0, $Type17 = "", $Param17 = 0, $Type18 = "", $Param18 = 0, $Type19 = "", $Param19 = 0, $Type20 = "", $Param20 = 0) ;assembles the code
    FasmReset($Fasm)
    If @error Then
        MsgBox(0, "_AssembleIt Error", "Error in Function FasmReset()")
        Return SetError(-2, 0, "ERROR -2")
    EndIf
    If $sFunc <> "" Then Call($sFunc)                          ;extract Assemblercode from function $sFunc()
    If @error = 0xDEAD Then
        MsgBox(0, "_AssembleIt Error", "The called function " & $sFunc & " doesnt exist or contains errors!")
        Return SetError(-4, 0, "ERROR -4")
    EndIf
    Local $bytecode = FasmGetBinary($Fasm)                     ;assemble ASM-code to opcodes
    If @extended Then                                          ;shows errors during assembling
        Local $Error = FasmGetLastError()                      ;gets errors
        MsgBox(0, "FASM-ERROR in function " & $sFunc & "()", "Error Code:" & $Error[0] & _
                @CRLF & "Error Message:" & $Error[1] & @CRLF & "Error Line:" & $Error[2] & @CRLF)
        Return SetError(-3, 0, "ERROR -3")
    Else                                                       ;no errors during assembling the code
        FileDelete("asm_test.bin")                             ;creates binary file with opcodes, in the case that someone wants to use an external debugger^^
        FileWrite("asm_test.bin", BinaryToString(String(FasmGetBinary($Fasm))))
        ConsoleWrite($bytecode & @CRLF)                        ;opcodes, can easily be copied and inserted somewhere....
        If $_ASSEMBLEIT_FLAG = 0 Then                          ;if less then 4 parameters, CallWindowProcW is possible
            If @NumParams > 10 Then                            ;only a maximum of 4 parameters in CallWindowProcW posssible
                MsgBox(0, "_AssembleIt Error", "The $_ASSEMBLEIT_FLAG is set to 0, but more than 4 Parameters are used in the Function " & $sFunc & @CRLF & _
                        "Please reduce the number of parameters to a maximum of 4 if you want an AutoItscript with a CallWindowProcW-call!")
                Exit
            Else                                               ;all is ready to create an AutoItscript which can execute the opcodes without FASM.au3
                Local $scriptstring = 'Global $tCodeBuffer = DllStructCreate("byte[' & StringLen($bytecode) / 2 - 1 & ']") ;reserve Memory for opcodes' & @CRLF & _
                        'DllStructSetData($tCodeBuffer, 1,"' & $bytecode & '") ;write opcodes into memory' & @CRLF
                $scriptstring &= '$ret=DllCall("user32.dll", "' & $Returntype & '",' & ' "CallWindowProcW", "ptr", DllStructGetPtr($tCodeBuffer)'
                Local $n = 1
                For $i = 3 To 9 Step 2                         ;CallWindowProcW must be called with 4 parameters...
                    $scriptstring &= ',"' & Eval("Type" & $n) & '",'
                    If Eval("Param" & $n) <> 0 Or Eval("Param" & $n) <> "" Then
                        $scriptstring &= "Param" & $n
                    Else
                        $scriptstring &= '0'
                    EndIf
                    $n += 1
                Next
                $scriptstring &= ')' & @CRLF
                ClipPut($scriptstring)                         ;puts the AutoItcode into Clipboard
                MsgBox(0, "_AssembleIt() Info!", "The following code was created and written into the Clipboard:" & _
                        @CRLF & @CRLF & $scriptstring & @CRLF & @CRLF & @CRLF & _
                        "This code can now be inserted into an AutoIt-Script, please adapt the parameters in the Dll-call to the used AutoIt-variables!" & _
                        @CRLF & "The Program will be terminated!")
                FasmExit($Fasm)
                Exit
            EndIf
        EndIf

        ;MemoryFuncCall
        Local $scriptstring = 'MemoryFuncCall("' & $Returntype & ':cdecl",' & FasmGetFuncPtr($Fasm) ;cdecl instructs the function to clean the stack, only a simple RET at the end is necessary
        ;Local $scriptstring = 'MemoryFuncCall("' & $Returntype & '",' & FasmGetFuncPtr($Fasm) ;if "compatible" mode to existing programs is required, please commend out this line
        Local $n = 1
        For $i = 3 To @NumParams Step 2                        ;all parameters
            $scriptstring &= ',"' & Eval("Type" & $n) & '", $Param' & $n
            $n += 1
        Next
        $scriptstring &= ')'
        Local $a = Execute($scriptstring)                      ;do the MemoryFuncCall, execute the opcodes
        If @error Then
            MsgBox(0, "_AssembleIt Error", "rror executing the MemoryFuncCall!")
            Return SetError(-5, 0, "ERROR -5")
        EndIf
        ;_arraydisplay($a)
        Return SetError(0, 0, $a[0])
    EndIf
EndFunc                                                        ;==>_AssembleIt

Func _($str)                                                   ;short version of Fasmadd
    Fasmadd($Fasm, $str)
EndFunc                                                        ;==>_


;debug-Fenster


Dim $_DBG_LABEL[170]

Global $hwnd_weiterbutton,$_DBG_closebutton
Global $_DBG_firstcall = True, $_DBG_buttonID

Global $_DBG_GUI = GUICreate("AssembleIt Debug-Info 1.0", 670, 550, 10, 10, 0, $WS_EX_DLGMODALFRAME)
$_DBG_winpos = WinGetPos($_DBG_GUI)
$_DBG_Window_posold_x = $_DBG_winpos[0]                        ;fensterposition merken
$_DBG_Window_posold_y = $_DBG_winpos[1] + $_DBG_winpos[3]

$WM_MOVING = 0x0216
GUIRegisterMsg($WM_MOVING, "_DBG_WM_MOVING")
GUIRegisterMsg($WM_SIZE, "_DBG_WM_SIZE")
;GUIRegisterMsg($WM_COMMAND, "_DBG_WM_COMMAND")


$_DBG_LABEL[18] = GUICtrlCreateLabel("FPU-Register showed as DOUBLE!", 10, 170, 290, 16)
GUICtrlSetFont(-1, -1, -1, 4)
;GUICtrlSetResizing ( -1, 32+ 2 )
$_DBG_LABEL[17] = GUICtrlCreateLabel("EFlags", 560, 16, 102, 16)
GUICtrlSetFont(-1, -1, -1, 4)

$_DBG_LABEL[38] = GUICtrlCreateLabel("CF =", 560, 32 + 16 * 0, 30, 16);CF
$_DBG_LABEL[59] = GUICtrlCreateLabel("DF =", 560, 32 + 16 * 1, 30, 16)
$_DBG_LABEL[39] = GUICtrlCreateLabel("PF =", 560, 32 + 16 * 2, 30, 16)
$_DBG_LABEL[68] = GUICtrlCreateLabel("OF =", 560, 32 + 16 * 3, 30, 16)
$_DBG_LABEL[48] = GUICtrlCreateLabel("AF =", 560, 32 + 16 * 4, 30, 16)
$_DBG_LABEL[49] = GUICtrlCreateLabel("ZF =", 560, 32 + 16 * 6, 30, 16)
$_DBG_LABEL[58] = GUICtrlCreateLabel("SF =", 560, 32 + 16 * 7, 30, 16)


For $i = 0 To 7
    $_DBG_LABEL[10 + $i] = GUICtrlCreateLabel("ST" & $i & " = ", 10 + Mod($i, 2) * 180, 190 + 16 * Int($i / 2), 30, 16)
    $_DBG_LABEL[80 + $i] = GUICtrlCreateLabel("", 50 + Mod($i, 2) * 180, 190 + 16 * Int($i / 2), 100, 16)
    $_DBG_LABEL[30 + $i] = GUICtrlCreateLabel("XMM" & $i & " = ", 10, 400 + 15 * $i, 40, 16);XMM0-XMM7
    $_DBG_LABEL[90 + $i] = GUICtrlCreateLabel("", 60, 400 + 15 * $i, 300, 16);XMM
    $_DBG_LABEL[40 + $i] = GUICtrlCreateLabel("", 60, 32 + 16 * $i, 400, 16);hex
    $_DBG_LABEL[50 + $i] = GUICtrlCreateLabel("", 150, 32 + 16 * $i, 400, 16);int
    $_DBG_LABEL[60 + $i] = GUICtrlCreateLabel("", 230, 32 + 16 * $i, 300, 16);float
    $_DBG_LABEL[70 + $i] = GUICtrlCreateLabel("", 320, 32 + 16 * $i, 240, 16);bin
    $_DBG_LABEL[100 + $i] = GUICtrlCreateLabel("", 590, 32 + 16 * $i, 40, 16) ;eflags
    $_DBG_LABEL[110 + $i] = GUICtrlCreateLabel("", 280, 400 + 15 * $i, 250, 16);XMM-2xdouble
    $_DBG_LABEL[120 + $i] = GUICtrlCreateLabel("", 440, 400 + 15 * $i, 250, 16);XMM-4xfloat
Next
GUICtrlSetPos($_DBG_LABEL[105], 590, 32 + 16 * 5, 1, 1)        ;platz machen fr ungenutztes label

$_DBG_LABEL[20] = GUICtrlCreateLabel("FPU-Flags", 10, 270, 55, 16)
GUICtrlSetFont(-1, -1, -1, 4)
$_DBG_LABEL[21] = GUICtrlCreateLabel("CO=            C1=            C2=", 520, 200, 135, 20)

$_DBG_LABEL[25] = GUICtrlCreateLabel("Stack", 370, 170, 130, 20)
GUICtrlSetFont(-1, -1, -1, 4)
$_DBG_LABEL[26] = GUICtrlCreateLabel("HEX", 450, 170, 130, 20)
GUICtrlSetFont(-1, -1, -1, 4)
$_DBG_LABEL[27] = GUICtrlCreateLabel("INT", 550, 170, 130, 20)
GUICtrlSetFont(-1, -1, -1, 4)
For $i = 40 To 0 Step -4
    $_DBG_LABEL[129 + $i / 4] = GUICtrlCreateLabel(StringFormat("[esp %+02.2d]", 40 - $i), 370, 190 + 16 * $i / 4, 50, 16);129-140
    $_DBG_LABEL[141 + $i / 4] = GUICtrlCreateLabel("", 450, 190 + 16 * $i / 4, 100, 16);141-152
    $_DBG_LABEL[155 + $i / 4] = GUICtrlCreateLabel("", 550, 190 + 16 * $i / 4, 100, 16);155-161
Next

$_DBG_LABEL[108] = GUICtrlCreateLabel("SSE-Register            HEX", 10, 375, 150, 20)
GUICtrlSetFont(-1, -1, -1, 4)
$_DBG_LABEL[109] = GUICtrlCreateLabel("2x Double", 280, 375, 100, 20)
GUICtrlSetFont(-1, -1, -1, 4)
$_DBG_LABEL[118] = GUICtrlCreateLabel("4x Float", 450, 375, 100, 20)
GUICtrlSetFont(-1, -1, -1, 4)


;GUIRegisterMsg($WM_COMMAND, "MyWM_COMMAND")

Global $listviewitem_reg32[8]
Global $reg_32[8] = ["EAX", "EBX", "ECX", "EDX", "ESI", "EDI", "ESP", "EBP"]

$Listview_reg32 = GUICtrlCreateListView("REG32|HEX|INT|FLOAT|BIN [BIT31....Bit0]", 10, 10, 520, 140, BitOR($GUI_SS_DEFAULT_LISTVIEW, $LVS_NOSORTHEADER));,$GUI_BKCOLOR_LV_ALTERNATE )
GUICtrlSetBkColor($Listview_reg32, 0xF0f0f0)                   ; Grau
GUICtrlSetBkColor($Listview_reg32, $GUI_BKCOLOR_LV_ALTERNATE)

_GUICtrlListView_BeginUpdate($Listview_reg32)
For $i = 0 To 7
    $listviewitem_reg32[$i] = GUICtrlCreateListViewItem($reg_32[$i] & "|0xDDDDDDDD|88888888888|9.99999999E999|00000000 00000000 00000000 00000000", $Listview_reg32)
    GUICtrlSetBkColor($listviewitem_reg32[$i], 0xFFFFFF)       ; weiss
Next
For $i = 0 To 4
    _GUICtrlListView_SetColumnWidth($Listview_reg32, $i, $LVSCW_AUTOSIZE_USEHEADER);$LVSCW_AUTOSIZE)
Next
_GUICtrlListView_EndUpdate($Listview_reg32)


;thx progandy fr den "button" !
Global Const $tagDLGTEMPLATE = "align 2                        ;DWORD style; DWORD dwExtendedStyle; WORD  cdit; short x; short y; short cx; short cy;"
Global Const $tagDLGITEMTEMPLATE = "align 2                    ;DWORD style; DWORD dwExtendedStyle; short x; short y; short cx; short cy; WORD  id;"



Global $_DBG_noshowflag=0
Global $dlgproc = DllCallbackRegister("_DlgProc", "bool", "hwnd;uint;wparam;lparam")
Global $_DBG_ = DllCallbackRegister("_DBG_MSGBOX", "dword", "dword;dword;dword;dword;dword;dword;dword;dword;dword")
;speicher reservieren fr datenbereich
Global $ptr_dbgmem = Number(_MemGlobalAlloc(600, 0))           ;512 byte + 8 byte weil nur 8byte-align

$mod = Mod($ptr_dbgmem, 16)                                    ;bentigt wird fr SSE-Register abe 16-byte-align
If $mod <> 0 Then
    $ptr_dbgmem += (16 - $mod)                                 ;16 byte align
EndIf


Global $_dbg_string[100], $_DBG_nr = 0, $_DBG_command[2]
Global $struct_FXSAVE = DllStructCreate("byte[512]", $ptr_dbgmem);platz fr Daten aus FXSAVE
Global $struct_STACK = DllStructCreate("dword[11]", $ptr_dbgmem + 520);platz fr Daten aus STACK [esp-20] bis [esp+20]
Global $ptr_STACK = DllStructGetPtr($struct_STACK)
;http://siyobik.info/index.php?module=x86&id=128
;ob ich diese flags noch einbaue, weiss ich nicht
$struct = DllStructCreate("" & _
        "word FCW;" & _                                        ;FPU control word  0+1
        "word FSW;" & _                                        ;FPU statusword    2+3
        "byte FTW;" & _                                        ;FPU ag word    4
        "byte;" & _                                            ;reserved    5
        "word FOP;" & _                                        ;FPU opcode    6+7
        "dword FIP;" & _                                       ;FPU instruction pointer 8-11
        "word CS;" & _                                         ;        12-13
        "word ;" & _                                           ;reserved    14-15
        "dword FDP;" & _                                       ; 16-19
        "word DS;" & _                                         ;    20+21
        "word ;" & _                                           ;reserved    22-23
        "dword MXCSR;" & _                                     ;MXCSR    24-27
        "dword MXCSR_MASK;" & _                                ;MXCSR_MASK    28-31
        "byte[10] ST0;")                                       ;ST0    32-41


$struct_double = DllStructCreate("double[8]")                  ;platz fr 8 doubles der FPU register st0-st7
$struct_128SSE = DllStructCreate("byte[128]", Ptr($ptr_dbgmem + 160));platz fr 16 byte SSE
$struct_EFLAGS = DllStructCreate("dword EFLAGS", Ptr($ptr_dbgmem + 512));platz 32 bit eflags
Global $ptr_SSE = DllStructGetPtr($struct_128SSE)              ;pointer
Global $ptr_EFLAGS = DllStructGetPtr($struct_EFLAGS)
$struct_SSE64x2int = DllStructCreate("uint64[16]", $ptr_SSE)   ;platz fr 2x 64byte SSE
$struct_SSE32x4int = DllStructCreate("uint[32]", $ptr_SSE)     ;platz fr 4x 32byte SSE
$struct_SSE16x8int = DllStructCreate("word[64]", $ptr_SSE)     ;platz fr 8x 16byte SSE
$struct_SSE64x2dbl = DllStructCreate("double[16]", $ptr_SSE)   ;platz fr 2x 64byte DOUBLE SSE
$struct_SSE32x4flt = DllStructCreate("float[32]", $ptr_SSE)    ;platz fr 4x 32byte FLOAT SSE



;debug-funktion, aus dem asmcode per call an die callback-adresse aufgerufen
Func _DBG_MSGBOX($anz, $edi, $esi, $ebp, $esp, $ebx, $edx, $ecx, $eax);aus asm bergebene register
	if $_DBG_noshowflag=1 then return 0
    GUISetState(@SW_SHOW, $_DBG_GUI)
    _WinAPI_UpdateWindow($_DBG_GUI)
    ;_DBG_WM_SIZE($_DBG_GUI,0,0,0)
    $pos_DBB_Window = WinGetPos($_DBG_GUI)                     ;Positionsdaten der GUI holen
    Local $x = Int($pos_DBB_Window[0] / 2)
    Local $y = Int(($pos_DBB_Window[1] + $pos_DBB_Window[3]) / 2)

    ;es folgen die Daten fr den "...NEXT"-Button, der muss ein modales Fenster sein, wer da eine andere Idee hat, bitte melden
    Local $w = Int($pos_DBB_Window[2] / 2)                     ;breite Button
    Local $h = 30                                              ;hhe Button
    $bTitle = StringToBinary("Next....", 2)                    ;text Button
    $bXY = BinaryMid(Binary($x), 1, 2) & BinaryMid(Binary($y), 1, 2);Buttondaten
    $bWH = BinaryMid(Binary($w), 1, 2) & BinaryMid(Binary($h), 1, 2)
    $bDIALOG = Binary("0x00000090400000040100") & $bXY & $bWH & Binary("0x000000000000") & Binary("0x000000500000000000000000") & $bWH & Binary("0x0100FFFF8000") & $bTitle & Binary("0x0000") & Binary("0x00000000")
    ;                   |Style ||ExStyl||cdit|                         |Empty "Arrays"|              |Style ||ExStyl||x ||y |                         |id||BUTTON|                       |Chr0|           |irgend welche anderen Arrays
    $tDLG = DllStructCreate("byte[" & BinaryLen($bDIALOG) & "]")
    DllStructSetData($tDLG, 1, $bDIALOG)                       ;Button-Daten in struct schreiben


    Dim $reg[8] = [$eax, $ebx, $ecx, $edx, $esi, $edi, $esp, $ebp]
    _GUICtrlListView_BeginUpdate($Listview_reg32)

    For $i = 0 To 7                                            ;fenster mit werten fllen
        GUICtrlSetData($listviewitem_reg32[$i], "|" & Ptr($reg[$i]) & "|" & _
                String($reg[$i]) & "|" & _
                StringFormat(" %2.6G", int2float($reg[$i])) & "|" & int2bin($reg[$i])) ;hex
        GUICtrlSetData($_DBG_LABEL[$i + 80], DllStructGetData($struct_double, 1, $i + 1));FPU st0-st7
        $struct = DllStructCreate("byte[16]", $ptr_SSE + 16 * $i)
        GUICtrlSetData($_DBG_LABEL[$i + 90], DllStructGetData($struct, 1))
        GUICtrlSetData($_DBG_LABEL[$i + 100], BitAND(2 ^ $i, DllStructGetData($struct_EFLAGS, 1)) / (2 ^ $i));eflags

        $struct = DllStructCreate("double[2]", $ptr_SSE + 16 * $i); 2x 64byte DOUBLE SSE
        GUICtrlSetData($_DBG_LABEL[$i + 110], StringFormat("%6s    %6s", DllStructGetData($struct, 1, 1), DllStructGetData($struct, 1, 2)))
        $struct = DllStructCreate("float[4]", $ptr_SSE + 16 * $i); 4x 32byte FLOAT SSE
        GUICtrlSetData($_DBG_LABEL[$i + 120], StringFormat("%10.5f  %10.5f  %10.5f  %10.5f", DllStructGetData($struct, 1, 1), DllStructGetData($struct, 1, 2), DllStructGetData($struct, 1, 4), DllStructGetData($struct, 1, 4)))
    Next
    GUICtrlSetData($_DBG_LABEL[101], BitAND(2 ^ 10, DllStructGetData($struct_EFLAGS, 1)) / (2 ^ 10));eflags DF
    GUICtrlSetData($_DBG_LABEL[103], BitAND(2 ^ 11, DllStructGetData($struct_EFLAGS, 1)) / (2 ^ 11));eflags OF
    For $i = 0 To 10                                           ;stack
        GUICtrlSetData($_DBG_LABEL[141 + $i], Ptr(DllStructGetData($struct_STACK, 1, $i + 1)));stack hex
        GUICtrlSetData($_DBG_LABEL[155 + $i], Int(DllStructGetData($struct_STACK, 1, $i + 1)));stack int
    Next
    _GUICtrlListView_EndUpdate($Listview_reg32)

    If $anz <> 0 Then
        If Execute($_dbg_string[$anz]) Then
            $ret = DllCall("user32.dll", "int", "DialogBoxIndirectParamW", "ptr", 0, "ptr", DllStructGetPtr($tDLG), "hwnd", 0, "ptr", DllCallbackGetPtr($dlgproc), "lparam", 0)
        EndIf
    Else
        ;DllCall("user32.dll", "int", "DialogBoxIndirectParamW", "ptr", 0, "ptr", DllStructGetPtr($tDLG), "hwnd", 0, "ptr", DllCallbackGetPtr($dlgproc), "lparam", 0)
        $ret = DllCall("user32.dll", "int", "DialogBoxIndirectParamW", "ptr", 0, "ptr", DllStructGetPtr($tDLG), "hwnd", 0, "ptr", DllCallbackGetPtr($dlgproc), "lparam", 0)
    EndIf
    ;    Exit
    Return 0
EndFunc                                                        ;==>_DBG_MSGBOX


;Alle Register, Flags, Stack usw werden in einen Speicherbereich geschrieben und von dort mit
;der Funktion _DBG_MSGBOX() ausgelesen
Func _asmdbg_($_DBG_command = "")                              ;Register

    _("push dword[esp+40]")                                    ;stack in memory
    _("pop dword[" & $ptr_STACK & "]")
    _("push dword[esp+36]")                                    ;stack in memory
    _("pop dword[" & $ptr_STACK + 4 & "]")
    _("push dword[esp+32]")                                    ;stack in memory
    _("pop dword[" & $ptr_STACK + 8 & "]")
    _("push dword[esp+28]")                                    ;stack in memory
    _("pop dword[" & $ptr_STACK + 12 & "]")
    _("push dword[esp+24]")                                    ;stack in memory
    _("pop dword[" & $ptr_STACK + 16 & "]")
    _("push dword[esp+20]")                                    ;stack in memory
    _("pop dword[" & $ptr_STACK + 20 & "]")
    _("push dword[esp+16]")                                    ;stack in memory
    _("pop dword[" & $ptr_STACK + 24 & "]")
    _("push dword[esp+12]")                                    ;stack in memory
    _("pop dword[" & $ptr_STACK + 28 & "]")
    _("push dword[esp+08]")                                    ;stack in memory
    _("pop dword[" & $ptr_STACK + 32 & "]")
    _("push dword[esp+04]")                                    ;stack in memory
    _("pop dword[" & $ptr_STACK + 36 & "]")
    _("push dword[esp+00]")                                    ;stack in memory
    _("pop dword[" & $ptr_STACK + 40 & "]")

    _("pushfd")                                                ;eflags sichern
    _("pop dword[" & Ptr($ptr_EFLAGS) & "]")                   ;eflags speichern in struct
    _("pushfd")                                                ;eflags sichern
    _("push eax")
    _("mov eax," & $ptr_dbgmem)                                ;alle FPU+SSE Registerinhalte und flags sichern
    _("FXSAVE [eax]")
    _("fstp qword[" & DllStructGetPtr($struct_double) & "]")   ;alle FPU-Register sichern
    _("fstp qword[" & DllStructGetPtr($struct_double) + 8 & "]")
    _("fstp qword[" & DllStructGetPtr($struct_double) + 16 & "]")
    _("fstp qword[" & DllStructGetPtr($struct_double) + 24 & "]")
    _("fstp qword[" & DllStructGetPtr($struct_double) + 32 & "]")
    _("fstp qword[" & DllStructGetPtr($struct_double) + 40 & "]")
    _("fstp qword[" & DllStructGetPtr($struct_double) + 48 & "]")
    _("fstp qword[" & DllStructGetPtr($struct_double) + 56 & "]")
    ;  _("fwait")
    _("pop eax")
    _("pushad")                                                ;alle Register sichern
    _("pushad")                                                ;auf den stack fr fr die msgbox
    If $_DBG_command <> "" Then                                ;falls kein Befehl bergeben wurde
        $_DBG_nr += 1
        $_dbg_string[$_DBG_nr] = $_DBG_command
        _("push " & Ptr($_DBG_nr))                             ;anzahl der
    Else
        _("push " & Ptr(0))
    EndIf
    _("call " & DllCallbackGetPtr($_DBG_))                     ;in autoit-callbackroutine springen
    _("mov eax," & $ptr_dbgmem)                                ;alle FPU+SSE registerinhalte und flags restore
    _("FXRSTOR [eax]")                                         ;restore alle FPU-Register
    ;  _("fwait")
    _("popad")                                                 ;alle register wieder zurcksetzen
    _("popfd")                                                 ;eflags setzen
EndFunc                                                        ;==>_asmdbg_

Func bin2ascii($bin_string)                                    ;string aus nullen und einsen in 8-bit-ascii text string umwandeln
    $step = 8                                                  ;8-Bit ASCII Buchstaben
    $ascii_string = ""                                         ;Rckgabestring
    For $f = 1 To StringLen($bin_string) Step $step            ;string von Vorne nach hinten 8-bitweise durchsuchen
        $t = StringMid($bin_string, $f, $step)                 ; 8-Bit-Wort, ein ASCII-Buchstabe
        $bin = 0                                               ;startwert fr
        For $i = 1 To $step                                    ;jedes Bit suchen
            If StringMid($t, $i, 1) = "1" Then $bin += (2 ^ ($step - $i)) ;wenn Bit=1 dann binrzahl=binrzahl+2^(8-Bitposition)
        Next
        $ascii_string &= Chr($bin)
    Next
    Return $ascii_string
EndFunc                                                        ;==>bin2ascii

Func int2bin($integer)                                         ;32Bit in binrstring darstellen
    $bin_string = ""
    For $i = 31 To 0 Step -1                                   ;asciicode in bits
        If Mod($i + 1, 8) = 0 Then $bin_string &= " "
        If BitAND($integer, 2 ^ $i) Then
            $bin_string &= "1"
        Else
            $bin_string &= "0"
        EndIf

    Next
    Return $bin_string
EndFunc                                                        ;==>int2bin

Func int2float($integer)
    Local $struct = DllStructCreate("int")
    Local $struct2 = DllStructCreate("float", DllStructGetPtr($struct))
    DllStructSetData($struct, 1, $integer)
    $ret = DllStructGetData($struct2, 1)
    $struct = 0
    $struct2 = 0
    Return $ret
EndFunc                                                        ;==>int2float

Func _DBG_WM_MOVING($hwnd, $message, $wParam, $lParam);fenster wird bewegt
    Local $pos = WinGetPos($_DBG_GUI)
    If $_DBG_Window_posold_x <> $pos[0] Or $_DBG_Window_posold_y <> $pos[1] Then ;fenster wurde bewegt
        $_DBG_Window_posold_x = $pos[0]                        ;fensterposition merken
        $_DBG_Window_posold_y = $pos[1]
        WinMove($hwnd_weiterbutton, "", $pos[0], $pos[1] + $pos[3], $pos[2], 60);buttonposition anpassen
    EndIf
    Return $GUI_RUNDEFMSG
EndFunc                                                        ;==>_DBG_WM_MOVING



Func _DBG_WM_SIZE($hwnd, $message, $wParam, $lParam);fenstergrsse wird verndert
    Local $posgui = WinGetPos($_DBG_GUI)
    $_DBG_Window_posold_x = $posgui[0]                         ;fensterposition merken
    $_DBG_Window_posold_y = $posgui[1]
    ;WinMove($hwnd_weiterbutton, "", $pos[0], $pos[1] + $pos[3], 200, 60);buttonposition anpassen
    WinMove($hwnd_weiterbutton, "", $posgui[0], $posgui[1] + $posgui[3], $posgui[2], 60);fensterposition anpassen
    WinMove($_DBG_GUI, "", $posgui[0], $posgui[1], $posgui[2], $posgui[3]);fensterposition anpassen
    _WinAPI_SetWindowPos($_DBG_buttonID, 0, 0, 0, $posgui[2], 60, 0x0020);buttonpos im fenster resze
EndFunc                                                        ;==>_DBG_WM_SIZE


Func _DlgProc($hwnd, $uMsg, $wParam, $lParam)
;thx to progandy!
    If $_DBG_firstcall Then
        $_DBG_buttonID = $wParam
        $_DBG_firstcall = False
    EndIf
    If $hwnd <> $_DBG_GUI Then $hwnd_weiterbutton = $hwnd
    If $uMsg = $WM_CLOSE Then
        DllCall("user32.dll", "bool", "EndDialog", "hwnd", $hwnd, "int_ptr", 0)
        Return True
    ElseIf $uMsg = $WM_COMMAND Then
        If BitAND($wParam, 0xFFFF) == 1 Then
            DllCall("user32.dll", "bool", "EndDialog", "hwnd", $hwnd, "int_ptr", 1)
            Return True
        EndIf
	elseif $uMsg = 123 then ;rechtsklick up schliesst das fenster, bzw returned sofort den callback, asm-prog luft weiter
		$_DBG_noshowflag=1
		DllCall("user32.dll", "bool", "EndDialog", "hwnd", $hwnd, "int_ptr", 0)
        Return True

    EndIf
    Return $GUI_RUNDEFMSG
EndFunc                                                        ;==>_DlgProc








